/*
** This is the OpenGL stuff for robotfindskitten3d
**
*/

#include <gl/gl.h>
#include <gl/glu.h>
#include <stdarg.h>
#include <stdio.h>
#include <strings.h>
#include <math.h>

#include "ogl.h"
#include "game.h"
#include "constants.h"

const float piover180 = 0.0174532925f;

extern void MakeFont( GLuint fontbase );
extern float GetTextLength( char *text );
extern void centreText( int y, char *str );
extern bool myLoadTexture(char *filename, my_texture *tex_obj);

extern int rfk_ticks;

extern screen_object robot;
extern screen_object kitten;
extern screen_object bogus[];
extern int num_bogus;

GLuint  fontbase; // base for font. fontbase. font - base. font/base.
my_texture scrolltex;
my_texture floortex;
my_texture hearttex;
//my_texture walltex;
my_texture gametex[ MAX_TEXTURES ];

float   zoom;

/*
** gl_Print
**
** Thanks to Jeff Molofee for this routine ( http://nehe.gamedev.net/ )
*/
void gl_Print( const char *fmt, ... )
{
    float   length;
    char    text[256];
    va_list ap;

    if( fmt == NULL ) return;

    va_start( ap, fmt );
      vsprintf( text, fmt, ap );
    va_end( ap );

    /* This is kind of OS specific */
    length = GetTextLength( text );

    glTranslatef( -length/2, 0.0f, 0.0f ); /* Centre our text */
    glPushAttrib( GL_LIST_BIT );
    glListBase( fontbase );
    glCallLists( strlen( text ), GL_UNSIGNED_BYTE, text );
    glPopAttrib();
    glTranslatef( -length/2, -1.0f, 0.0f ); /* get back to centre, next line */
}

/*
** gl_FPRint
**
** A little bit quicker... (no varargs)
*/
void gl_FPrint( char *str, float length, int len )
{
    glTranslatef( length, 0.0f, 0.0f );
    glPushAttrib( GL_LIST_BIT );
    glListBase( fontbase );
    glCallLists( len, GL_UNSIGNED_BYTE, str );
    glPopAttrib();
    glTranslatef( length, -1.0f, 0.0f );
}

/*
** Set up a font to use with OpenGL
*/
bool gl_InitFont( void )
{

    // Create lists for our font
    fontbase = glGenLists(256);

    // Do OS specific font initialisation
    MakeFont( fontbase );
    return true;
}

/*
** Set up OpenGL
*/
bool gl_Init( void )
{

    int t;
    char tn[40];

	glViewport(0,0,W_WIDTH,W_HEIGHT);						// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(GLfloat)W_WIDTH/W_HEIGHT,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix

	glEnable(GL_TEXTURE_2D);							// Enable Texture Mapping
	glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);				// Black Background
	glClearDepth(1.0f);									// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);								// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations
	glEnable(GL_LIGHT0);								// Enable Default Light (Quick And Dirty)
	glEnable(GL_LIGHTING);								// Enable Lighting
	glEnable(GL_COLOR_MATERIAL);						// Enable Coloring Of Material

	if( myLoadTexture( "instructions.bmp", &scrolltex ) == false ) return false;
	if( myLoadTexture( "floor.bmp", &floortex ) == false ) return false;
	if( myLoadTexture( "heart.bmp", &hearttex ) == false ) return false;
//	if( myLoadTexture( "wall.bmp", &walltex ) == false ) return false;
	
	for( t=0; t<MAX_TEXTURES; t++ )
	{
        sprintf( tn, "texture%d.bmp", t );
        if( myLoadTexture( tn, &gametex[t] ) == false ) return false;
	}

    return gl_InitFont();
}

/*
** Kill OpenGL!
*/
void gl_Kill( void )
{
    glDeleteLists( fontbase, 256 );
}

/*
** Draw a screen_object
*/
void gl_DrawObject( screen_object *o )
{
    glPushMatrix();     // remember where we are
    glTranslatef( o->x, 0, o->y );
    glRotatef( o->ang, 0.0f, 1.0f, 0.0f );
   	glBindTexture( GL_TEXTURE_2D, gametex[ o->texture ].num );
    glBegin( GL_QUADS );
		// Front Face
		glNormal3f( 0.0f, 0.0f, 1.0f);					// Normal Pointing Towards Viewer
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-0.5f, -0.5f,  0.5f);	// Point 1 (Front)
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 0.5f, -0.5f,  0.5f);	// Point 2 (Front)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 0.5f,  0.5f,  0.5f);	// Point 3 (Front)
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-0.5f,  0.5f,  0.5f);	// Point 4 (Front)
		// Back Face
		glNormal3f( 0.0f, 0.0f,-1.0f);					// Normal Pointing Away From Viewer
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-0.5f, -0.5f, -0.5f);	// Point 1 (Back)
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-0.5f,  0.5f, -0.5f);	// Point 2 (Back)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 0.5f,  0.5f, -0.5f);	// Point 3 (Back)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 0.5f, -0.5f, -0.5f);	// Point 4 (Back)
		// Top Face
		glNormal3f( 0.0f, 1.0f, 0.0f);					// Normal Pointing Up
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-0.5f,  0.5f, -0.5f);	// Point 1 (Top)
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-0.5f,  0.5f,  0.5f);	// Point 2 (Top)
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 0.5f,  0.5f,  0.5f);	// Point 3 (Top)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 0.5f,  0.5f, -0.5f);	// Point 4 (Top)

		// Bottom Face (never seen)
//		glNormal3f( 0.0f,-1.0f, 0.0f);					// Normal Pointing Down
//		glTexCoord2f(1.0f, 1.0f); glVertex3f(-0.5f, -0.5f, -0.5f);	// Point 1 (Bottom)
//		glTexCoord2f(0.0f, 1.0f); glVertex3f( 0.5f, -0.5f, -0.5f);	// Point 2 (Bottom)
//		glTexCoord2f(0.0f, 0.0f); glVertex3f( 0.5f, -0.5f,  0.5f);	// Point 3 (Bottom)
//		glTexCoord2f(1.0f, 0.0f); glVertex3f(-0.5f, -0.5f,  0.5f);	// Point 4 (Bottom)

		// Right face
		glNormal3f( 1.0f, 0.0f, 0.0f);					// Normal Pointing Right
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 0.5f, -0.5f, -0.5f);	// Point 1 (Right)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 0.5f,  0.5f, -0.5f);	// Point 2 (Right)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 0.5f,  0.5f,  0.5f);	// Point 3 (Right)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 0.5f, -0.5f,  0.5f);	// Point 4 (Right)
		// Left Face
		glNormal3f(-1.0f, 0.0f, 0.0f);					// Normal Pointing Left
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-0.5f, -0.5f, -0.5f);	// Point 1 (Left)
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-0.5f, -0.5f,  0.5f);	// Point 2 (Left)
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-0.5f,  0.5f,  0.5f);	// Point 3 (Left)
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-0.5f,  0.5f, -0.5f);	// Point 4 (Left)
	glEnd();								// Done Drawing Quads
	glPopMatrix();    
}

/*
** Draw the zen plane
*/
void gl_DrawScene( float camx, float camy )
{
    int i;
    glLoadIdentity();
    glRotatef( 20.0f, 1.0f, 0.0f, 0.0f );
    glRotatef( -robot.ang, 0.0f, 1.0f, 0.0f );

    glTranslatef( -robot.x-camx, -1.7f*zoom, -robot.y-camy );

    // Draw the floor
    glBindTexture( GL_TEXTURE_2D, floortex.num );
    glBegin( GL_QUADS );
      glNormal3f( 0.0f, 1.0f, 0.0f );
      glTexCoord2f(    0.0f,     0.0f ); glVertex3f(        -0.5f, -0.5f,         -0.5f );
      glTexCoord2f( G_WIDTH,     0.0f ); glVertex3f( G_WIDTH-0.5f, -0.5f,         -0.5f );
      glTexCoord2f( G_WIDTH, G_HEIGHT ); glVertex3f( G_WIDTH-0.5f, -0.5f, G_HEIGHT-0.5f );
      glTexCoord2f(    0.0f, G_HEIGHT ); glVertex3f(        -0.5f, -0.5f, G_HEIGHT-0.5f );
    glEnd();

/*
        // Draw the walls
        glBindTexture( GL_TEXTURE_2D, walltex.num );
        glBegin( GL_QUADS );
          glNormal3f( 0.0f, 0.0f, 1.0f );
          glTexCoord2f(    0.0f, 0.0f ); glVertex3f(        -0.5f,  4.5f, -0.5f );
          glTexCoord2f( G_WIDTH, 0.0f ); glVertex3f( G_WIDTH-0.5f,  4.5f, -0.5f );
          glTexCoord2f( G_WIDTH, 5.0f ); glVertex3f( G_WIDTH-0.5f, -0.5f, -0.5f );
          glTexCoord2f(    0.0f, 5.0f ); glVertex3f(        -0.5f, -0.5f, -0.5f );

          glNormal3f( 0.0f, 0.0f, -1.0f );
          glTexCoord2f(    0.0f, 0.0f ); glVertex3f(        -0.5f,  4.5f, G_HEIGHT-0.5f );
          glTexCoord2f( G_WIDTH, 0.0f ); glVertex3f( G_WIDTH-0.5f,  4.5f, G_HEIGHT-0.5f );
          glTexCoord2f( G_WIDTH, 5.0f ); glVertex3f( G_WIDTH-0.5f, -0.5f, G_HEIGHT-0.5f );
          glTexCoord2f(    0.0f, 5.0f ); glVertex3f(        -0.5f, -0.5f, G_HEIGHT-0.5f );

        glEnd();
*/

    gl_DrawObject( &robot );
    gl_DrawObject( &kitten );
    for( i=0; i<num_bogus; i++ ) gl_DrawObject( &bogus[ i ] );
}


/*
** Draw everything
*/
void gl_Draw( int doing )
{

    float insanim,camx,camy;
    msg_bogus tnki;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear Screen And Depth Buffer

    camx = (float)sin( robot.ang*piover180 ) * 3.0f;
    camy = (float)cos( robot.ang*piover180 ) * 3.0f;

    switch( doing ) {
    case DOING_WIN_MESSAGE:
    
        if( zoom < 12.0f )
        {
            zoom += 0.05f;
            if( zoom == 12.0f ) rfk_ticks = 0;
            
            camx *= zoom;
            camy *= zoom;

            gl_DrawScene( camx, camy );
        } else {
            glLoadIdentity();

            insanim = (float)rfk_ticks / 300 + 1.0f;
            if( insanim > 5 )
            {
                insanim = 5.0f;
                rfkEndGame();
            } 
            
            glBindTexture( GL_TEXTURE_2D, hearttex.num );
            glBegin( GL_QUADS );
              glNormal3f( 0.0f, 0.0f, 1.0f );
              glTexCoord2f( 0.0f, 0.0f ); glVertex3f( -insanim, -insanim, -12.0f );
              glTexCoord2f( 1.0f, 0.0f ); glVertex3f(  insanim, -insanim, -12.0f );
              glTexCoord2f( 1.0f, 1.0f ); glVertex3f(  insanim,  insanim, -12.0f );
              glTexCoord2f( 0.0f, 1.0f ); glVertex3f( -insanim,  insanim, -12.0f );
            glEnd();

            robot.ang = 0;
            insanim = 3 - (((float)rfk_ticks) / 200);
            if( insanim < 0 ) insanim = 0.0f;    

            robot.x = -(insanim+0.5f);
            robot.y = -10;
            kitten.x = insanim+0.5f;
            kitten.y = -10;

            gl_DrawObject( &robot );
            gl_DrawObject( &kitten );

            if( rfk_ticks < 500 )
            {
              glLoadIdentity();
              glTranslatef( 0.0f, -2.0f, -8.0f );
              glBindTexture( GL_TEXTURE_2D, gametex[ 2 ].num );
              glTexCoord2f( 0.0f, 0.0f );
              gl_Print( "YOU FOUND KITTEN!" );
            } else {
              glLoadIdentity();
              glTranslatef( 0.0f, -2.0f, -8.0f );
              glBindTexture( GL_TEXTURE_2D, gametex[ 2 ].num );
              glTexCoord2f( 0.0f, 0.30f );
              gl_Print( "WAY TO GO, ROBOT!" );            
            }

        }
        break;        

    case DOING_MESSAGE:

        insanim = sin( ((float)rfk_ticks) / 5 ) * 1.2;
        glLoadIdentity();
        glTranslatef( 0.0f, 0.0f, -20.0f+insanim );
        if( zoom < 6.0f ) zoom += 0.2f;
        glBindTexture( GL_TEXTURE_2D, gametex[ 1 ].num );
        glTexCoord2f( 0.0f, 0.0f );
        tnki = rfkGetNki();
        gl_FPrint( tnki.str, tnki.width, tnki.len );

    case DOING_GAME:

        if(( doing == DOING_GAME ) && ( zoom > 1.0f )) zoom -= 0.2f;

        camx *= zoom;
        camy *= zoom;

        gl_DrawScene( camx, camy );
        break;

    case DOING_INSTRUCTIONS:
    
        insanim = ((float)(rfk_ticks%8000))/400 - 13;
    
        glLoadIdentity();
        glTranslatef( 0.0f, 0.0f, -5.0f );
        glRotatef( -60.0f, 1.0f, 0.0f, 0.0f );
        glTranslatef( 0.0f, insanim, 0.0f );
       	glBindTexture( GL_TEXTURE_2D, scrolltex.num );
       	glBegin( GL_QUADS );
            // Front Face
            glNormal3f( 0.0f, 0.0f, 1.0f);
            glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f,  0.0f,  1.0f);
            glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f,  0.0f,  1.0f);
            glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f, 10.0f,  1.0f);
            glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f, 10.0f,  1.0f);
        glEnd();       	  

        break;
    }
}


