/**************************

  robotfindskitten3d
  
  windows.c
  
  all the non-portable crap in here.
  (god I hate the Win32 API)

 **************************/

#include <windows.h>
#include <stdio.h>
#include <gl/gl.h>
#include <gl/glu.h>

#include "ogl.h"
#include "game.h"
#include "constants.h"

/**************************
 * Function Declarations
 *
 **************************/

LRESULT CALLBACK WndProc (HWND hWnd, UINT message,
WPARAM wParam, LPARAM lParam);
void EnableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC);
void DisableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC);
void MakeFont( GLuint fontbase );
float GetTextLength( char *text );
bool myLoadTexture(char *filename, my_texture *tex_obj);

extern int doing;

/* This should not be global, but it is. So sue me. */
HDC global_hDC;
HWND global_hWnd;
GLYPHMETRICSFLOAT gmf[256];	// Storage For Information About Our Font
int num_texture = 0;

#if DEBUG_LEVEL > DEBUG_NONE
FILE *debug_log;
#endif

/* OS Specifc font initialisation */
void MakeFont( GLuint fontbase )
{

    HFONT font;

    font = CreateFont( -8,         // 8 PIXELS (positive numbers for cells)
                        0,         // default width
                        0,         // Angle of escapement
                        0,         // Orientation angle
                       FW_NORMAL,
                       FALSE,       // Italic
                       FALSE,       // Underline
                       FALSE,       // Strikeout
                       ANSI_CHARSET,
                       OUT_RASTER_PRECIS,
                       CLIP_DEFAULT_PRECIS,
                       DRAFT_QUALITY,
                       FF_DONTCARE|DEFAULT_PITCH,
                       "Arial Narrow");
    
    SelectObject( global_hDC, font );

    wglUseFontOutlines( global_hDC, 0, 255, fontbase, 0.0f, 0.2f, WGL_FONT_POLYGONS, gmf );
}

/* OS Specific text lengthification routine */
/* lengthification is a word I just made up. I think its great. */
float GetTextLength( char *text )
{
    float length = 0;

    for( unsigned int loop = 0; loop < (strlen(text)); loop++ )
        length += gmf[ text[ loop ] ].gmfCellIncX;

    return length;
}

/* Almost portable, but requires windows.h :( */
bool myLoadTexture(char *filename, my_texture *tex_obj) 
{ 

   unsigned char *l_texture;
   int l_index, l_index2=0;
   FILE *file;
   BITMAPFILEHEADER fileheader; 
   BITMAPINFOHEADER infoheader;
   RGBTRIPLE rgb; 

   num_texture++;

   if( (file = fopen(filename, "rb"))==NULL) return false; 

   fread(&fileheader, sizeof(fileheader), 1, file); 
   fseek(file, sizeof(fileheader), SEEK_SET);
   fread(&infoheader, sizeof(infoheader), 1, file);

   l_texture = (byte *) malloc(infoheader.biWidth * infoheader.biHeight * 3);
   memset(l_texture, 0, infoheader.biWidth * infoheader.biHeight * 3);

   for (l_index=0; l_index < infoheader.biWidth*infoheader.biHeight; l_index++)
   { 
      fread(&rgb, sizeof(rgb), 1, file); 

      l_texture[l_index2+0] = rgb.rgbtRed; // Red component
      l_texture[l_index2+1] = rgb.rgbtGreen; // Green component
      l_texture[l_index2+2] = rgb.rgbtBlue; // Blue component
      l_index2 += 3; // Go to the next position
   }

   fclose(file); // Closes the file stream

   glBindTexture(GL_TEXTURE_2D, num_texture);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
   gluBuild2DMipmaps(GL_TEXTURE_2D, 3, infoheader.biWidth, infoheader.biHeight, GL_RGB, GL_UNSIGNED_BYTE, l_texture);

   tex_obj->width  = infoheader.biWidth;
   tex_obj->height = infoheader.biHeight;
   tex_obj->num    = num_texture;
   return true;
}

/**************************
 * WinMain
 *
 **************************/

int WINAPI 
WinMain (HINSTANCE hInstance,
         HINSTANCE hPrevInstance,
         LPSTR lpCmdLine,
         int iCmdShow)
{
    WNDCLASS wc;
    HGLRC hRC;        
    MSG msg;
    BOOL bQuit = FALSE;
    float theta = 0.0f;

#if DEBUG_LEVEL > DEBUG_NONE
    debug_log = fopen( "debug.log", "w" );
    if( debug_log == NULL ) return 0;
#endif

    /* register window class */
    wc.style = CS_OWNDC;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor (NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH) GetStockObject (BLACK_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "rfk3d";
    RegisterClass (&wc);

    /* create main window */
    global_hWnd = CreateWindow (
      "rfk3d", "robotfindskitten3d", 
      WS_CAPTION | WS_POPUPWINDOW | WS_VISIBLE,
      0, 0, W_WIDTH, W_HEIGHT,
      NULL, NULL, hInstance, NULL);

    /* enable OpenGL for the window */
    EnableOpenGL ( global_hWnd, &global_hDC, &hRC );

    /* Initialise everything */
    if( rfkInitGame() )
    {

        /* program main loop */
        while (!bQuit)
        {
            /* check for messages */
            if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE))
            {
            
                /* handle or dispatch messages */
                if (msg.message == WM_QUIT)
                {
                    bQuit = TRUE;
                }
                else
                {
                    TranslateMessage (&msg);
                    DispatchMessage (&msg);
                }
            }
            else
            {
                gl_Draw( doing );
                rfkGameTick();                // This *should* be called under a timer!!
				SwapBuffers(global_hDC);					// Swap Buffers (Double Buffering)
            }
        }

        /* General shutdown code */
        rfkShutDown();
    }
    
    /* shutdown OpenGL */
    DisableOpenGL ( global_hWnd, global_hDC, hRC );

    /* destroy the window explicitly */
    DestroyWindow ( global_hWnd );

#if DEBUG_LEVEL > DEBUG_NONE
    fclose( debug_log );
#endif

    return msg.wParam;
}


/********************
 * Window Procedure
 *
 ********************/

LRESULT CALLBACK 
WndProc (HWND hWnd, UINT message,
         WPARAM wParam, LPARAM lParam)
{

    switch (message)
    {
    case WM_CREATE:
        return 0;
    case WM_CLOSE:
        PostQuitMessage (0);
        return 0;

    case WM_DESTROY:
        return 0;

    case WM_KEYUP:

        switch (wParam)        // deal with windows events
        {
        case VK_UP:
            wParam = 'Q';      // Translate cursor keys into QAOP (very ZX spectrum :)
            break;
        case VK_DOWN:
            wParam = 'A';
            break;
        case VK_LEFT:
            wParam = 'O';
            break;
        case VK_RIGHT:
            wParam = 'P';
            break;
        }

        if( (( wParam >= 'A' ) && ( wParam <= 'Z' )) ||
            (( wParam >= 'a' ) && ( wParam <= 'z' )) ||
            (( wParam >= '0' ) && ( wParam <= '9' )) ||
            ( wParam == 27 ) || ( wParam == ' ' ) ) rfkProcessKey( wParam, false );

        return 0;

    case WM_KEYDOWN:
#if DEBUG_LEVEL >= DEBUG_VERBOSE
        char mystr[80];
        sprintf( mystr, "Got key: %ld\n", wParam );
        fputs( mystr, debug_log );
#endif

        switch (wParam)        // deal with windows events
        {
        case VK_UP:
            wParam = 'Q';      // Translate cursor keys into QAOP (very ZX spectrum :)
            break;
        case VK_DOWN:
            wParam = 'A';
            break;
        case VK_LEFT:
            wParam = 'O';
            break;
        case VK_RIGHT:
            wParam = 'P';
            break;
        }

        if( (( wParam >= 'A' ) && ( wParam <= 'Z' )) ||
            (( wParam >= 'a' ) && ( wParam <= 'z' )) ||
            (( wParam >= '0' ) && ( wParam <= '9' )) ||
            ( wParam == 27 ) || ( wParam == ' ' ) ) rfkProcessKey( wParam, true );

        return 0;

    default:
        return DefWindowProc (hWnd, message, wParam, lParam);
    }
}


/*******************
 * Enable OpenGL
 *
 *******************/

void
EnableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC)
{
    PIXELFORMATDESCRIPTOR pfd;
    int iFormat;

    /* get the device context (DC) */
    *hDC = GetDC (hWnd);

    /* set the pixel format for the DC */
    ZeroMemory (&pfd, sizeof (pfd));
    pfd.nSize = sizeof (pfd);
    pfd.nVersion = 1;
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | 
      PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
    pfd.iPixelType = PFD_TYPE_RGBA;
    pfd.cColorBits = 24;
    pfd.cDepthBits = 16;
    pfd.iLayerType = PFD_MAIN_PLANE;
    iFormat = ChoosePixelFormat (*hDC, &pfd);
    SetPixelFormat (*hDC, iFormat, &pfd);

    /* create and enable the render context (RC) */
    *hRC = wglCreateContext( *hDC );
    wglMakeCurrent( *hDC, *hRC );

}


/******************
 * Disable OpenGL
 *
 ******************/

void
DisableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC)
{
    wglMakeCurrent (NULL, NULL);
    wglDeleteContext (hRC);
    ReleaseDC (hWnd, hDC);
}

